EXPECTED = {'X691-A3': {'extensibility-implied': False,
             'imports': {},
             'object-classes': {},
             'object-sets': {},
             'types': {'ChildInformation': {'members': [{'name': 'name',
                                                         'type': 'Name'},
                                                        {'name': 'dateOfBirth',
                                                         'tag': {'number': 0},
                                                         'type': 'Date'},
                                                        '...',
                                                        {'name': 'sex',
                                                         'optional': True,
                                                         'tag': {'kind': 'IMPLICIT',
                                                                 'number': 1},
                                                         'type': 'ENUMERATED',
                                                         'values': [('male', 1),
                                                                    ('female',
                                                                     2),
                                                                    ('unknown',
                                                                     3)]}],
                                            'type': 'SET'},
                       'Date': {'tag': {'class': 'APPLICATION',
                                        'kind': 'IMPLICIT',
                                        'number': 3},
                                'type': 'VisibleString'},
                       'EmployeeNumber': {'restricted-to': [(0, 9999), '...'],
                                          'tag': {'class': 'APPLICATION',
                                                  'kind': 'IMPLICIT',
                                                  'number': 2},
                                          'type': 'INTEGER'},
                       'Name': {'members': [{'name': 'givenName',
                                             'type': 'NameString'},
                                            {'name': 'initial',
                                             'type': 'NameString'},
                                            {'name': 'familyName',
                                             'type': 'NameString'},
                                            '...'],
                                'tag': {'class': 'APPLICATION',
                                        'kind': 'IMPLICIT',
                                        'number': 1},
                                'type': 'SEQUENCE'},
                       'NameString': {'type': 'VisibleString'},
                       'PersonnelRecord': {'members': [{'name': 'name',
                                                        'type': 'Name'},
                                                       {'name': 'title',
                                                        'tag': {'number': 0},
                                                        'type': 'VisibleString'},
                                                       {'name': 'number',
                                                        'type': 'EmployeeNumber'},
                                                       {'name': 'dateOfHire',
                                                        'tag': {'number': 1},
                                                        'type': 'Date'},
                                                       {'name': 'nameOfSpouse',
                                                        'tag': {'number': 2},
                                                        'type': 'Name'},
                                                       {'element': {'type': 'ChildInformation'},
                                                        'name': 'children',
                                                        'optional': True,
                                                        'size': [2, '...'],
                                                        'tag': {'kind': 'IMPLICIT',
                                                                'number': 3},
                                                        'type': 'SEQUENCE OF'},
                                                       '...'],
                                           'tag': {'class': 'APPLICATION',
                                                   'kind': 'IMPLICIT',
                                                   'number': 0},
                                           'type': 'SET'}},
             'values': {}}}