EXPECTED = {'ITS-Container': {'extensibility-implied': False,
                   'imports': {},
                   'object-classes': {},
                   'object-sets': {},
                   'tags': 'AUTOMATIC',
                   'types': {'AccelerationConfidence': {'restricted-to': [(0,
                                                                           102)],
                                                        'type': 'INTEGER'},
                             'AccelerationControl': {'size': [7],
                                                     'type': 'BIT STRING'},
                             'AccidentSubCauseCode': {'restricted-to': [(0,
                                                                         255)],
                                                      'type': 'INTEGER'},
                             'ActionID': {'members': [{'name': 'originatingStationID',
                                                       'type': 'StationID'},
                                                      {'name': 'sequenceNumber',
                                                       'type': 'SequenceNumber'}],
                                          'type': 'SEQUENCE'},
                             'AdverseWeatherCondition-AdhesionSubCauseCode': {'restricted-to': [(0,
                                                                                                 255)],
                                                                              'type': 'INTEGER'},
                             'AdverseWeatherCondition-ExtremeWeatherConditionSubCauseCode': {'restricted-to': [(0,
                                                                                                                255)],
                                                                                             'type': 'INTEGER'},
                             'AdverseWeatherCondition-PrecipitationSubCauseCode': {'restricted-to': [(0,
                                                                                                      255)],
                                                                                   'type': 'INTEGER'},
                             'AdverseWeatherCondition-VisibilitySubCauseCode': {'restricted-to': [(0,
                                                                                                   255)],
                                                                                'type': 'INTEGER'},
                             'Altitude': {'members': [{'name': 'altitudeValue',
                                                       'type': 'AltitudeValue'},
                                                      {'name': 'altitudeConfidence',
                                                       'type': 'AltitudeConfidence'}],
                                          'type': 'SEQUENCE'},
                             'AltitudeConfidence': {'type': 'ENUMERATED',
                                                    'values': [('alt-000-01',
                                                                0),
                                                               ('alt-000-02',
                                                                1),
                                                               ('alt-000-05',
                                                                2),
                                                               ('alt-000-10',
                                                                3),
                                                               ('alt-000-20',
                                                                4),
                                                               ('alt-000-50',
                                                                5),
                                                               ('alt-001-00',
                                                                6),
                                                               ('alt-002-00',
                                                                7),
                                                               ('alt-005-00',
                                                                8),
                                                               ('alt-010-00',
                                                                9),
                                                               ('alt-020-00',
                                                                10),
                                                               ('alt-050-00',
                                                                11),
                                                               ('alt-100-00',
                                                                12),
                                                               ('alt-200-00',
                                                                13),
                                                               ('outOfRange',
                                                                14),
                                                               ('unavailable',
                                                                15)]},
                             'AltitudeValue': {'restricted-to': [(-100000,
                                                                  800001)],
                                               'type': 'INTEGER'},
                             'CauseCode': {'members': [{'name': 'causeCode',
                                                        'type': 'CauseCodeType'},
                                                       {'name': 'subCauseCode',
                                                        'type': 'SubCauseCodeType'}],
                                           'type': 'SEQUENCE'},
                             'CauseCodeType': {'restricted-to': [(0, 255)],
                                               'type': 'INTEGER'},
                             'CenDsrcTollingZone': {'members': [{'name': 'protectedZoneLatitude',
                                                                 'type': 'Latitude'},
                                                                {'name': 'protectedZoneLongitude',
                                                                 'type': 'Longitude'},
                                                                {'name': 'cenDsrcTollingZoneID',
                                                                 'optional': True,
                                                                 'type': 'CenDsrcTollingZoneID'}],
                                                    'type': 'SEQUENCE'},
                             'CenDsrcTollingZoneID': {'type': 'ProtectedZoneID'},
                             'ClosedLanes': {'members': [{'name': 'hardShoulderStatus',
                                                          'optional': True,
                                                          'type': 'HardShoulderStatus'},
                                                         {'name': 'drivingLaneStatus',
                                                          'type': 'DrivingLaneStatus'},
                                                         '...'],
                                             'type': 'SEQUENCE'},
                             'CollisionRiskSubCauseCode': {'restricted-to': [(0,
                                                                              255)],
                                                           'type': 'INTEGER'},
                             'Curvature': {'members': [{'name': 'curvatureValue',
                                                        'type': 'CurvatureValue'},
                                                       {'name': 'curvatureConfidence',
                                                        'type': 'CurvatureConfidence'}],
                                           'type': 'SEQUENCE'},
                             'CurvatureCalculationMode': {'type': 'ENUMERATED',
                                                          'values': [('yawRateUsed',
                                                                      0),
                                                                     ('yawRateNotUsed',
                                                                      1),
                                                                     ('unavailable',
                                                                      2),
                                                                     '...']},
                             'CurvatureConfidence': {'type': 'ENUMERATED',
                                                     'values': [('onePerMeter-0-00002',
                                                                 0),
                                                                ('onePerMeter-0-0001',
                                                                 1),
                                                                ('onePerMeter-0-0005',
                                                                 2),
                                                                ('onePerMeter-0-002',
                                                                 3),
                                                                ('onePerMeter-0-01',
                                                                 4),
                                                                ('onePerMeter-0-1',
                                                                 5),
                                                                ('outOfRange',
                                                                 6),
                                                                ('unavailable',
                                                                 7)]},
                             'CurvatureValue': {'restricted-to': [(-30000,
                                                                   30001)],
                                                'type': 'INTEGER'},
                             'DangerousEndOfQueueSubCauseCode': {'restricted-to': [(0,
                                                                                    255)],
                                                                 'type': 'INTEGER'},
                             'DangerousGoodsBasic': {'type': 'ENUMERATED',
                                                     'values': [('explosives1',
                                                                 0),
                                                                ('explosives2',
                                                                 1),
                                                                ('explosives3',
                                                                 2),
                                                                ('explosives4',
                                                                 3),
                                                                ('explosives5',
                                                                 4),
                                                                ('explosives6',
                                                                 5),
                                                                ('flammableGases',
                                                                 6),
                                                                ('nonFlammableGases',
                                                                 7),
                                                                ('toxicGases',
                                                                 8),
                                                                ('flammableLiquids',
                                                                 9),
                                                                ('flammableSolids',
                                                                 10),
                                                                ('substancesLiableToSpontaneousCombustion',
                                                                 11),
                                                                ('substancesEmittingFlammableGasesUponContactWithWater',
                                                                 12),
                                                                ('oxidizingSubstances',
                                                                 13),
                                                                ('organicPeroxides',
                                                                 14),
                                                                ('toxicSubstances',
                                                                 15),
                                                                ('infectiousSubstances',
                                                                 16),
                                                                ('radioactiveMaterial',
                                                                 17),
                                                                ('corrosiveSubstances',
                                                                 18),
                                                                ('miscellaneousDangerousSubstances',
                                                                 19)]},
                             'DangerousGoodsExtended': {'members': [{'name': 'dangerousGoodsType',
                                                                     'type': 'DangerousGoodsBasic'},
                                                                    {'name': 'unNumber',
                                                                     'restricted-to': [(0,
                                                                                        9999)],
                                                                     'type': 'INTEGER'},
                                                                    {'name': 'elevatedTemperature',
                                                                     'type': 'BOOLEAN'},
                                                                    {'name': 'tunnelsRestricted',
                                                                     'type': 'BOOLEAN'},
                                                                    {'name': 'limitedQuantity',
                                                                     'type': 'BOOLEAN'},
                                                                    {'name': 'emergencyActionCode',
                                                                     'optional': True,
                                                                     'size': [(1,
                                                                               24)],
                                                                     'type': 'IA5String'},
                                                                    {'name': 'phoneNumber',
                                                                     'optional': True,
                                                                     'size': [(1,
                                                                               24)],
                                                                     'type': 'IA5String'},
                                                                    {'name': 'companyName',
                                                                     'optional': True,
                                                                     'size': [(1,
                                                                               24)],
                                                                     'type': 'UTF8String'}],
                                                        'type': 'SEQUENCE'},
                             'DangerousSituationSubCauseCode': {'restricted-to': [(0,
                                                                                   255)],
                                                                'type': 'INTEGER'},
                             'DeltaAltitude': {'restricted-to': [(-12700,
                                                                  12800)],
                                               'type': 'INTEGER'},
                             'DeltaLatitude': {'restricted-to': [(-131071,
                                                                  131072)],
                                               'type': 'INTEGER'},
                             'DeltaLongitude': {'restricted-to': [(-131071,
                                                                   131072)],
                                                'type': 'INTEGER'},
                             'DeltaReferencePosition': {'members': [{'name': 'deltaLatitude',
                                                                     'type': 'DeltaLatitude'},
                                                                    {'name': 'deltaLongitude',
                                                                     'type': 'DeltaLongitude'},
                                                                    {'name': 'deltaAltitude',
                                                                     'type': 'DeltaAltitude'}],
                                                        'type': 'SEQUENCE'},
                             'DriveDirection': {'type': 'ENUMERATED',
                                                'values': [('forward', 0),
                                                           ('backward', 1),
                                                           ('unavailable', 2)]},
                             'DrivingLaneStatus': {'size': [(1, 14)],
                                                   'type': 'BIT STRING'},
                             'EmbarkationStatus': {'type': 'BOOLEAN'},
                             'EmergencyPriority': {'size': [2],
                                                   'type': 'BIT STRING'},
                             'EmergencyVehicleApproachingSubCauseCode': {'restricted-to': [(0,
                                                                                            255)],
                                                                         'type': 'INTEGER'},
                             'EnergyStorageType': {'size': [7],
                                                   'type': 'BIT STRING'},
                             'EventHistory': {'element': {'type': 'EventPoint'},
                                              'size': [(1, 23)],
                                              'type': 'SEQUENCE OF'},
                             'EventPoint': {'members': [{'name': 'eventPosition',
                                                         'type': 'DeltaReferencePosition'},
                                                        {'name': 'eventDeltaTime',
                                                         'optional': True,
                                                         'type': 'PathDeltaTime'},
                                                        {'name': 'informationQuality',
                                                         'type': 'InformationQuality'}],
                                            'type': 'SEQUENCE'},
                             'ExteriorLights': {'size': [8],
                                                'type': 'BIT STRING'},
                             'HardShoulderStatus': {'type': 'ENUMERATED',
                                                    'values': [('availableForStopping',
                                                                0),
                                                               ('closed', 1),
                                                               ('availableForDriving',
                                                                2)]},
                             'HazardousLocation-AnimalOnTheRoadSubCauseCode': {'restricted-to': [(0,
                                                                                                  255)],
                                                                               'type': 'INTEGER'},
                             'HazardousLocation-DangerousCurveSubCauseCode': {'restricted-to': [(0,
                                                                                                 255)],
                                                                              'type': 'INTEGER'},
                             'HazardousLocation-ObstacleOnTheRoadSubCauseCode': {'restricted-to': [(0,
                                                                                                    255)],
                                                                                 'type': 'INTEGER'},
                             'HazardousLocation-SurfaceConditionSubCauseCode': {'restricted-to': [(0,
                                                                                                   255)],
                                                                                'type': 'INTEGER'},
                             'Heading': {'members': [{'name': 'headingValue',
                                                      'type': 'HeadingValue'},
                                                     {'name': 'headingConfidence',
                                                      'type': 'HeadingConfidence'}],
                                         'type': 'SEQUENCE'},
                             'HeadingConfidence': {'restricted-to': [(1, 127)],
                                                   'type': 'INTEGER'},
                             'HeadingValue': {'restricted-to': [(0, 3601)],
                                              'type': 'INTEGER'},
                             'HeightLonCarr': {'restricted-to': [(1, 100)],
                                               'type': 'INTEGER'},
                             'HumanPresenceOnTheRoadSubCauseCode': {'restricted-to': [(0,
                                                                                       255)],
                                                                    'type': 'INTEGER'},
                             'HumanProblemSubCauseCode': {'restricted-to': [(0,
                                                                             255)],
                                                          'type': 'INTEGER'},
                             'InformationQuality': {'restricted-to': [(0, 7)],
                                                    'type': 'INTEGER'},
                             'ItineraryPath': {'element': {'type': 'ReferencePosition'},
                                               'size': [(1, 40)],
                                               'type': 'SEQUENCE OF'},
                             'ItsPduHeader': {'members': [{'name': 'protocolVersion',
                                                           'restricted-to': [(0,
                                                                              255)],
                                                           'type': 'INTEGER'},
                                                          {'name': 'messageID',
                                                           'restricted-to': [(0,
                                                                              255)],
                                                           'type': 'INTEGER'},
                                                          {'name': 'stationID',
                                                           'type': 'StationID'}],
                                              'type': 'SEQUENCE'},
                             'LanePosition': {'restricted-to': [(-1, 14)],
                                              'type': 'INTEGER'},
                             'LateralAcceleration': {'members': [{'name': 'lateralAccelerationValue',
                                                                  'type': 'LateralAccelerationValue'},
                                                                 {'name': 'lateralAccelerationConfidence',
                                                                  'type': 'AccelerationConfidence'}],
                                                     'type': 'SEQUENCE'},
                             'LateralAccelerationValue': {'restricted-to': [(-160,
                                                                             161)],
                                                          'type': 'INTEGER'},
                             'Latitude': {'restricted-to': [(-900000000,
                                                             900000001)],
                                          'type': 'INTEGER'},
                             'LightBarSirenInUse': {'size': [2],
                                                    'type': 'BIT STRING'},
                             'Longitude': {'restricted-to': [(-1800000000,
                                                              1800000001)],
                                           'type': 'INTEGER'},
                             'LongitudinalAcceleration': {'members': [{'name': 'longitudinalAccelerationValue',
                                                                       'type': 'LongitudinalAccelerationValue'},
                                                                      {'name': 'longitudinalAccelerationConfidence',
                                                                       'type': 'AccelerationConfidence'}],
                                                          'type': 'SEQUENCE'},
                             'LongitudinalAccelerationValue': {'restricted-to': [(-160,
                                                                                  161)],
                                                               'type': 'INTEGER'},
                             'NumberOfOccupants': {'restricted-to': [(0, 127)],
                                                   'type': 'INTEGER'},
                             'PathDeltaTime': {'restricted-to': [(1, 65535),
                                                                 '...'],
                                               'type': 'INTEGER'},
                             'PathHistory': {'element': {'type': 'PathPoint'},
                                             'size': [(0, 40)],
                                             'type': 'SEQUENCE OF'},
                             'PathPoint': {'members': [{'name': 'pathPosition',
                                                        'type': 'DeltaReferencePosition'},
                                                       {'name': 'pathDeltaTime',
                                                        'optional': True,
                                                        'type': 'PathDeltaTime'}],
                                           'type': 'SEQUENCE'},
                             'PerformanceClass': {'restricted-to': [(0, 7)],
                                                  'type': 'INTEGER'},
                             'PosCentMass': {'restricted-to': [(1, 63)],
                                             'type': 'INTEGER'},
                             'PosConfidenceEllipse': {'members': [{'name': 'semiMajorConfidence',
                                                                   'type': 'SemiAxisLength'},
                                                                  {'name': 'semiMinorConfidence',
                                                                   'type': 'SemiAxisLength'},
                                                                  {'name': 'semiMajorOrientation',
                                                                   'type': 'HeadingValue'}],
                                                      'type': 'SEQUENCE'},
                             'PosFrontAx': {'restricted-to': [(1, 20)],
                                            'type': 'INTEGER'},
                             'PosLonCarr': {'restricted-to': [(1, 127)],
                                            'type': 'INTEGER'},
                             'PosPillar': {'restricted-to': [(1, 30)],
                                           'type': 'INTEGER'},
                             'PositionOfOccupants': {'size': [20],
                                                     'type': 'BIT STRING'},
                             'PositionOfPillars': {'element': {'type': 'PosPillar'},
                                                   'size': [(1, 3), '...'],
                                                   'type': 'SEQUENCE OF'},
                             'PositioningSolutionType': {'type': 'ENUMERATED',
                                                         'values': [('noPositioningSolution',
                                                                     0),
                                                                    ('sGNSS',
                                                                     1),
                                                                    ('dGNSS',
                                                                     2),
                                                                    ('sGNSSplusDR',
                                                                     3),
                                                                    ('dGNSSplusDR',
                                                                     4),
                                                                    ('dR', 5),
                                                                    '...']},
                             'PostCrashSubCauseCode': {'restricted-to': [(0,
                                                                          255)],
                                                       'type': 'INTEGER'},
                             'ProtectedCommunicationZone': {'members': [{'name': 'protectedZoneType',
                                                                         'type': 'ProtectedZoneType'},
                                                                        {'name': 'expiryTime',
                                                                         'optional': True,
                                                                         'type': 'TimestampIts'},
                                                                        {'name': 'protectedZoneLatitude',
                                                                         'type': 'Latitude'},
                                                                        {'name': 'protectedZoneLongitude',
                                                                         'type': 'Longitude'},
                                                                        {'name': 'protectedZoneRadius',
                                                                         'optional': True,
                                                                         'type': 'ProtectedZoneRadius'},
                                                                        {'name': 'protectedZoneID',
                                                                         'optional': True,
                                                                         'type': 'ProtectedZoneID'}],
                                                            'type': 'SEQUENCE'},
                             'ProtectedCommunicationZonesRSU': {'element': {'type': 'ProtectedCommunicationZone'},
                                                                'size': [(1,
                                                                          16)],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                             'ProtectedZoneID': {'restricted-to': [(0,
                                                                    134217727)],
                                                 'type': 'INTEGER'},
                             'ProtectedZoneRadius': {'restricted-to': [(1, 255),
                                                                       '...'],
                                                     'type': 'INTEGER'},
                             'ProtectedZoneType': {'type': 'ENUMERATED',
                                                   'values': [('cenDsrcTolling',
                                                               0),
                                                              '...']},
                             'PtActivation': {'members': [{'name': 'ptActivationType',
                                                           'type': 'PtActivationType'},
                                                          {'name': 'ptActivationData',
                                                           'type': 'PtActivationData'}],
                                              'type': 'SEQUENCE'},
                             'PtActivationData': {'size': [(1, 20)],
                                                  'type': 'OCTET STRING'},
                             'PtActivationType': {'restricted-to': [(0, 255)],
                                                  'type': 'INTEGER'},
                             'ReferencePosition': {'members': [{'name': 'latitude',
                                                                'type': 'Latitude'},
                                                               {'name': 'longitude',
                                                                'type': 'Longitude'},
                                                               {'name': 'positionConfidenceEllipse',
                                                                'type': 'PosConfidenceEllipse'},
                                                               {'name': 'altitude',
                                                                'type': 'Altitude'}],
                                                   'type': 'SEQUENCE'},
                             'RelevanceDistance': {'type': 'ENUMERATED',
                                                   'values': [('lessThan50m',
                                                               0),
                                                              ('lessThan100m',
                                                               1),
                                                              ('lessThan200m',
                                                               2),
                                                              ('lessThan500m',
                                                               3),
                                                              ('lessThan1000m',
                                                               4),
                                                              ('lessThan5km',
                                                               5),
                                                              ('lessThan10km',
                                                               6),
                                                              ('over10km', 7)]},
                             'RelevanceTrafficDirection': {'type': 'ENUMERATED',
                                                           'values': [('allTrafficDirections',
                                                                       0),
                                                                      ('upstreamTraffic',
                                                                       1),
                                                                      ('downstreamTraffic',
                                                                       2),
                                                                      ('oppositeTraffic',
                                                                       3)]},
                             'RequestResponseIndication': {'type': 'ENUMERATED',
                                                           'values': [('request',
                                                                       0),
                                                                      ('response',
                                                                       1)]},
                             'RescueAndRecoveryWorkInProgressSubCauseCode': {'restricted-to': [(0,
                                                                                                255)],
                                                                             'type': 'INTEGER'},
                             'RestrictedTypes': {'element': {'type': 'StationType'},
                                                 'size': [(1, 3), '...'],
                                                 'type': 'SEQUENCE OF'},
                             'RoadType': {'type': 'ENUMERATED',
                                          'values': [('urban-NoStructuralSeparationToOppositeLanes',
                                                      0),
                                                     ('urban-WithStructuralSeparationToOppositeLanes',
                                                      1),
                                                     ('nonUrban-NoStructuralSeparationToOppositeLanes',
                                                      2),
                                                     ('nonUrban-WithStructuralSeparationToOppositeLanes',
                                                      3)]},
                             'RoadworksSubCauseCode': {'restricted-to': [(0,
                                                                          255)],
                                                       'type': 'INTEGER'},
                             'SemiAxisLength': {'restricted-to': [(0, 4095)],
                                                'type': 'INTEGER'},
                             'SequenceNumber': {'restricted-to': [(0, 65535)],
                                                'type': 'INTEGER'},
                             'SignalViolationSubCauseCode': {'restricted-to': [(0,
                                                                                255)],
                                                             'type': 'INTEGER'},
                             'SlowVehicleSubCauseCode': {'restricted-to': [(0,
                                                                            255)],
                                                         'type': 'INTEGER'},
                             'SpecialTransportType': {'size': [4],
                                                      'type': 'BIT STRING'},
                             'Speed': {'members': [{'name': 'speedValue',
                                                    'type': 'SpeedValue'},
                                                   {'name': 'speedConfidence',
                                                    'type': 'SpeedConfidence'}],
                                       'type': 'SEQUENCE'},
                             'SpeedConfidence': {'restricted-to': [(1, 127)],
                                                 'type': 'INTEGER'},
                             'SpeedLimit': {'restricted-to': [(1, 255)],
                                            'type': 'INTEGER'},
                             'SpeedValue': {'restricted-to': [(0, 16383)],
                                            'type': 'INTEGER'},
                             'StationID': {'restricted-to': [(0, 4294967295)],
                                           'type': 'INTEGER'},
                             'StationType': {'restricted-to': [(0, 255)],
                                             'type': 'INTEGER'},
                             'StationarySince': {'type': 'ENUMERATED',
                                                 'values': [('lessThan1Minute',
                                                             0),
                                                            ('lessThan2Minutes',
                                                             1),
                                                            ('lessThan15Minutes',
                                                             2),
                                                            ('equalOrGreater15Minutes',
                                                             3)]},
                             'StationaryVehicleSubCauseCode': {'restricted-to': [(0,
                                                                                  255)],
                                                               'type': 'INTEGER'},
                             'SteeringWheelAngle': {'members': [{'name': 'steeringWheelAngleValue',
                                                                 'type': 'SteeringWheelAngleValue'},
                                                                {'name': 'steeringWheelAngleConfidence',
                                                                 'type': 'SteeringWheelAngleConfidence'}],
                                                    'type': 'SEQUENCE'},
                             'SteeringWheelAngleConfidence': {'restricted-to': [(1,
                                                                                 127)],
                                                              'type': 'INTEGER'},
                             'SteeringWheelAngleValue': {'restricted-to': [(-511,
                                                                            512)],
                                                         'type': 'INTEGER'},
                             'SubCauseCodeType': {'restricted-to': [(0, 255)],
                                                  'type': 'INTEGER'},
                             'Temperature': {'restricted-to': [(-60, 67)],
                                             'type': 'INTEGER'},
                             'TimestampIts': {'restricted-to': [(0,
                                                                 4398046511103)],
                                              'type': 'INTEGER'},
                             'Traces': {'element': {'type': 'PathHistory'},
                                        'size': [(1, 7)],
                                        'type': 'SEQUENCE OF'},
                             'TrafficConditionSubCauseCode': {'restricted-to': [(0,
                                                                                 255)],
                                                              'type': 'INTEGER'},
                             'TrafficRule': {'type': 'ENUMERATED',
                                             'values': [('noPassing', 0),
                                                        ('noPassingForTrucks',
                                                         1),
                                                        ('passToRight', 2),
                                                        ('passToLeft', 3),
                                                        '...']},
                             'TransmissionInterval': {'restricted-to': [(1,
                                                                         10000)],
                                                      'type': 'INTEGER'},
                             'TurningRadius': {'restricted-to': [(1, 255)],
                                               'type': 'INTEGER'},
                             'VDS': {'size': [6], 'type': 'IA5String'},
                             'ValidityDuration': {'restricted-to': [(0, 86400)],
                                                  'type': 'INTEGER'},
                             'VehicleBreakdownSubCauseCode': {'restricted-to': [(0,
                                                                                 255)],
                                                              'type': 'INTEGER'},
                             'VehicleIdentification': {'members': [{'name': 'wMInumber',
                                                                    'optional': True,
                                                                    'type': 'WMInumber'},
                                                                   {'name': 'vDS',
                                                                    'optional': True,
                                                                    'type': 'VDS'},
                                                                   '...'],
                                                       'type': 'SEQUENCE'},
                             'VehicleLength': {'members': [{'name': 'vehicleLengthValue',
                                                            'type': 'VehicleLengthValue'},
                                                           {'name': 'vehicleLengthConfidenceIndication',
                                                            'type': 'VehicleLengthConfidenceIndication'}],
                                               'type': 'SEQUENCE'},
                             'VehicleLengthConfidenceIndication': {'type': 'ENUMERATED',
                                                                   'values': [('noTrailerPresent',
                                                                               0),
                                                                              ('trailerPresentWithKnownLength',
                                                                               1),
                                                                              ('trailerPresentWithUnknownLength',
                                                                               2),
                                                                              ('trailerPresenceIsUnknown',
                                                                               3),
                                                                              ('unavailable',
                                                                               4)]},
                             'VehicleLengthValue': {'restricted-to': [(1,
                                                                       1023)],
                                                    'type': 'INTEGER'},
                             'VehicleMass': {'restricted-to': [(1, 1024)],
                                             'type': 'INTEGER'},
                             'VehicleRole': {'type': 'ENUMERATED',
                                             'values': [('default', 0),
                                                        ('publicTransport', 1),
                                                        ('specialTransport', 2),
                                                        ('dangerousGoods', 3),
                                                        ('roadWork', 4),
                                                        ('rescue', 5),
                                                        ('emergency', 6),
                                                        ('safetyCar', 7),
                                                        ('agriculture', 8),
                                                        ('commercial', 9),
                                                        ('military', 10),
                                                        ('roadOperator', 11),
                                                        ('taxi', 12),
                                                        ('reserved1', 13),
                                                        ('reserved2', 14),
                                                        ('reserved3', 15)]},
                             'VehicleWidth': {'restricted-to': [(1, 62)],
                                              'type': 'INTEGER'},
                             'VerticalAcceleration': {'members': [{'name': 'verticalAccelerationValue',
                                                                   'type': 'VerticalAccelerationValue'},
                                                                  {'name': 'verticalAccelerationConfidence',
                                                                   'type': 'AccelerationConfidence'}],
                                                      'type': 'SEQUENCE'},
                             'VerticalAccelerationValue': {'restricted-to': [(-160,
                                                                              161)],
                                                           'type': 'INTEGER'},
                             'WMInumber': {'size': [(1, 3)],
                                           'type': 'IA5String'},
                             'WheelBaseVehicle': {'restricted-to': [(1, 127)],
                                                  'type': 'INTEGER'},
                             'WrongWayDrivingSubCauseCode': {'restricted-to': [(0,
                                                                                255)],
                                                             'type': 'INTEGER'},
                             'YawRate': {'members': [{'name': 'yawRateValue',
                                                      'type': 'YawRateValue'},
                                                     {'name': 'yawRateConfidence',
                                                      'type': 'YawRateConfidence'}],
                                         'type': 'SEQUENCE'},
                             'YawRateConfidence': {'type': 'ENUMERATED',
                                                   'values': [('degSec-000-01',
                                                               0),
                                                              ('degSec-000-05',
                                                               1),
                                                              ('degSec-000-10',
                                                               2),
                                                              ('degSec-001-00',
                                                               3),
                                                              ('degSec-005-00',
                                                               4),
                                                              ('degSec-010-00',
                                                               5),
                                                              ('degSec-100-00',
                                                               6),
                                                              ('outOfRange', 7),
                                                              ('unavailable',
                                                               8)]},
                             'YawRateValue': {'restricted-to': [(-32766,
                                                                 32767)],
                                              'type': 'INTEGER'}},
                   'values': {}}}