EXPECTED = {'Bar': {'extensibility-implied': False,
         'imports': {},
         'object-classes': {},
         'object-sets': {},
         'tags': 'EXPLICIT',
         'types': {'AcceptTypes': {'members': [{'element': {'type': 'BIT '
                                                                    'STRING'},
                                                'name': 'standardTypes',
                                                'optional': True,
                                                'tag': {'kind': 'IMPLICIT',
                                                        'number': 0},
                                                'type': 'SEQUENCE OF'},
                                               {'element': {'type': 'OCTET '
                                                                    'STRING'},
                                                'name': 'otherTypes',
                                                'optional': True,
                                                'tag': {'kind': 'IMPLICIT',
                                                        'number': 1},
                                                'type': 'SEQUENCE OF'}],
                                   'tag': {'class': 'APPLICATION',
                                           'kind': 'IMPLICIT',
                                           'number': 1},
                                   'type': 'SEQUENCE'},
                   'GetRequest': {'members': [{'name': 'headerOnly',
                                               'type': 'BOOLEAN'},
                                              {'name': 'lock',
                                               'type': 'BOOLEAN'},
                                              {'name': 'acceptTypes',
                                               'optional': True,
                                               'type': 'AcceptTypes'},
                                              {'name': 'url',
                                               'type': 'OCTET STRING'}],
                                  'tag': {'class': 'APPLICATION',
                                          'kind': 'IMPLICIT',
                                          'number': 0},
                                  'type': 'SEQUENCE'}},
         'values': {}}}